"""Главный файл приложения"""

from os import getenv
from dotenv import load_dotenv
from telethon import TelegramClient
from textual.app import App
from textual.keys import Keys, _character_to_key
from datetime import timezone, timedelta
from src.screens import AuthScreen, ChatScreen
import src.locales

load_dotenv()
env_consts = ["API_ID", "API_HASH", "LANGUAGE", "UTC_OFFSET", "CHATS_LIMIT",
              "MESSAGES_LIMIT"]
for name in env_consts:
    exec(f"{name} = getenv(\"{name}\")")

if "" in list(map(lambda x: globals()[x], env_consts)):
    raise ValueError(
        "Недостаточно параметров в .env файле."
        "Скопируйте .env.example в .env и заполните свои API-ключи."
        "Not enough settings in .env file."
        "Copy .env.example into .env and fill your API-keys."
    )

API_ID = int(API_ID) # type: ignore
locale = dict(zip(
    getattr(src.locales, "codes"), 
    getattr(src.locales, LANGUAGE) # type: ignore
))

class Talc(App):
    """Класс приложения"""

    CSS_PATH = "style.tcss"
    BINDINGS = [
        (Keys.ControlE, "notify(\"Нажата кнопка профиля\")", locale["you"]),
        (Keys.ControlF, "notify(\"Нажата кнопка папок\")", locale["folders"]),
        (Keys.Tab, "notify(\"Нажат таб\")", locale["switch_focus"]),
        (Keys.Enter, "notify(\"Нажат энтер\")", locale["enter"]),
        (Keys.Escape, "show_start_screen()", locale["back"]),
        (_character_to_key("/"), "notify(\"Нажат слэш\")", locale["search"])
    ]

    def __init__(
        self, 
        driver_class = None, 
        css_path = None, 
        watch_css = False, 
        ansi_color = False
    ):
        super().__init__(driver_class, css_path, watch_css, ansi_color)
        self.locale = locale
        self.title = self.locale["talc"]
        self.timezone = timezone(timedelta(hours=int(UTC_OFFSET))) # type: ignore
        self.CHATS_LIMIT = CHATS_LIMIT # type: ignore
        self.MESSAGES_LIMIT = MESSAGES_LIMIT # type: ignore
        self.msg_buffer = None

    async def on_mount(self) -> None:
        self.telegram_client = TelegramClient(
            getenv("CURRENT_USER"), 
            API_ID, 
            API_HASH # type: ignore
        )
        await self.telegram_client.connect()

        self.chat_screen = ChatScreen(telegram_client=self.telegram_client)
        self.install_screen(self.chat_screen, name="chats")

        if not await self.telegram_client.is_user_authorized():
            auth_screen = AuthScreen(telegram_client=self.telegram_client)
            self.install_screen(auth_screen, name="auth")
            self.push_screen("auth")
        else:
            self.push_screen("chats")

        self.scroll_sensitivity_y = 1.0

    def show_start_screen(self) -> None:
        self.chat_screen.switcher.current = "start_label"

    async def on_exit_app(self):
        await self.telegram_client.disconnect()
        return super()._on_exit_app()
